@@ -1,5 +1,4 @@
 package File::Remove;
-require 5.004; # just because I think you should upgrade :)
 
 =head1 NAME
 
@@ -9,25 +8,27 @@ B<File::Remove> - Remove files and directories
 
     use File::Remove qw(remove);
 
-    remove \1,"file1","file2","directory1","file3","directory2","file4",
-        "directory2","file5","directory3";
+    # removes (without recursion) several files
+    remove qw( *.c *.pl );
 
-    # removes only files, even if the filespec matches a directory
-    remove "*.c","*.pl";
+    # removes (with recursion) several directories
+    remove \1, qw( directory1 directory2 ); 
 
-    # recurses into subdirectories and removes them all
-    remove \1, "directory"; 
+    # removes (with recursion) several files and directories
+    remove \1, qw( file1 file2 directory1 *~ );
 
 =head1 DESCRIPTION
 
 B<File::Remove::remove> removes files and directories.  It acts like
-B<rm> for the most part.  Although unlink can be given a list of
-files it will not remove directories.  This module remedies that.  It
-also accepts wildcards, * and ?, as arguments for filenames.
+B<rm>, for the most part.  Although unlink can be given a list of files,
+it will not remove directories; this module remedies that.  It also
+accepts wildcards, * and ?, as arguments for filenames.
+
+=head1 METHODS
 
 =over 4
 
-=item B<remove>
+=item X<remove>
 
 Removes files and directories.  Directories are removed recursively like
 in B<rm -rf> if the first argument is a reference to a scalar that
@@ -38,28 +39,27 @@ In list context it returns a list of files/directories removed, in
 scalar context it returns the number of files/directories removed.  The
 list/number should match what was passed in if everything went well.
 
-=item B<rm>
+=item X<rm>
 
 Just calls remove.  It's there for people who get tired of typing
 'remove'.
 
 =back
 
-=head1 EXAMPLE
-
-See SYNOPSIS.
-
 =head1 BUGS
 
-Not that I know of. ;)
+See http://rt.cpan.org/NoAuth/Bugs.html?Dist=File-Remove for the
+up-to-date bug listing.
 
 =head1 AUTHOR
 
-Gabor Egressy B<gabor@vmunix.com>
+Taken over by Richard Soderberg, E<lt>perl@crystalflame.netE<gt>, so as
+to port it to L<File::Spec>.
 
-Copyright (c) 1998 Gabor Egressy.  All rights reserved.  All wrongs
-reversed.  This program is free software; you can redistribute and/or
-modify it under the same terms as Perl itself.
+Original copyright: (c) 1998 by Gabor Egressy, E<lt>gabor@vmunix.comE<gt>.
+
+All rights reserved.  All wrongs reversed.  This program is free software;
+you can redistribute and/or modify it under the same terms as Perl itself.
 
 =cut
 
@@ -69,7 +69,9 @@ use vars qw(@EXPORT_OK @ISA $VERSION $debug);
 # we export nothing by default :)
 @EXPORT_OK = qw(remove rm);
 
-$VERSION = '0.20';
+use File::Spec;
+
+$VERSION = '0.21';
 
 sub _recurse_dir($);
 
@@ -80,24 +82,25 @@ sub _recurse_dir($)
     chmod 0777,$dir;
     opendir DIR,$dir
         or return 0;
-    my @files = grep {! /^\.\.?$/} readdir DIR
+    my @files = File::Spec->no_upwards(readdir DIR)
         or return 0;
     closedir DIR;
 
     my $ret;
     for (@files) {
-        print "$dir/$_\n"
+	my $file = File::Spec->catfile($dir, $_);
+	# TODO: this needs to be more aware of catdir
+        print "file: $file\n"
             if $debug;
-        if(-f "$dir/$_" || -l "$dir/$_") {
-            unlink $dir . '/' . $_
+        if(-f $file || -l $file) {
+            unlink $file
                 or next;
             $ret = 1;
-        }
-        elsif(-d "$dir/$_" && ! -l "$dir/$_") {
-            _recurse_dir $dir . '/' . $_;
-            rmdir $dir . '/' . $_
-                or next;
-        }
+        } elsif (-d $file && ! -l $file) {
+	    _recurse_dir $file;
+	    rmdir $file
+	        or next;
+	}
     }
     $ret;
 }
@@ -107,7 +110,7 @@ sub expand(@)
     my @args;
 
     for (@_) {
-        push @args,glob;
+        push @args, glob;
     }
     @args;
 }
@@ -129,20 +132,25 @@ sub remove(@)
 
     my $ret;
     for (@files) {
-        print "$_\n"
-            if $debug;
+        print "file: $_\n" if $debug;
         if(-f $_ || -l $_) {
+            print "file unlink: $_\n" if $debug;
             unlink $_
                 and push @removes,$_;
         }
         elsif(-d $_ && $$recursive) {
+	    print "dir: $_\n" if $debug;
+	    # XXX: this regex seems unnecessary, and may trigger bugs someday.
+	    # TODO: but better to trim trailing slashes for now.
             s/\/$//;
             my ($save_mode) = (stat $_)[2];
             $ret = _recurse_dir $_;
             chmod $save_mode & 0777,$_; # just in case we cannot remove it.
             rmdir $_
                 and push @removes, $_;
-        }
+        } else {
+	    print "???: $_\n" if $debug;
+	}
     }
 
     @removes;
